// SPDX-License-Identifier: GPL-2.0-only
/*
 *  add audit function based on extend attribute security.seaudit
 *
 *  This file contains the selinux hooks2 function implementations.
 *
 *  Authors:  zhoupeng, <zhoupeng@uniontech.com>
 *
 *  Copyright (C) 2022 UnionTech Technology, Inc.
 */
#ifndef __LINUX_LSM_HOOKS2_H
#define __LINUX_LSM_HOOKS2_H
#include <linux/security2.h>

#define LSM_HOOK2_INIT(HEAD, HOOK) \
	{ .head = &security_hook2_heads.HEAD, .hook = { .HEAD = HOOK } }

// add hook for selinux
union security_list2_options {
	int (*file_close2)(struct file* file);
	int (*file_write2)(struct file* file, size_t count);
	int (*file_read2)(struct file* file, size_t count);
	int (*process_exit2)(struct task_struct * task);
	int (*inode_init_security2)(struct inode *inode, struct inode *dir,
                    const struct qstr *qstr,
                    const char **name, void **value,
                    size_t *len);

	int (*file_create2)(const struct inode* dir_inode, struct dentry* dentryy, SECURITY_E_TYPE op);
	int (*socket_getpeersec2_stream)(struct socket *sock, char __user *optval,
					int __user *optlen, unsigned len);

	/* add extensible hooks for net monitor */
	int (* nm_tcp_recvmsg)(struct sock * sk, struct msghdr *msg, size_t copied);
	int (* nm_udp_recvmsg)(struct sock * sk, struct msghdr *msg, size_t copied);
};

struct security_hook2_heads {
	struct hlist_head file_close2;
	struct hlist_head file_write2;
	struct hlist_head file_read2;
	struct hlist_head process_exit2;
	struct hlist_head inode_init_security2;
	struct hlist_head file_create2;
	struct hlist_head socket_getpeersec2_stream;

	/* add extensible hooks */
	struct hlist_head nm_tcp_recvmsg;
	struct hlist_head nm_udp_recvmsg;
};

extern struct security_hook2_heads security_hook2_heads;

struct security_hook2_list {
	struct hlist_node       list;
	struct hlist_head       *head;
	union security_list2_options    hook;
	char                *lsm;
} __randomize_layout;


extern void security_add_hooks2(struct security_hook2_list *hooks, int count,
                char *lsm);

static inline void security_delete_hooks2(struct security_hook2_list *hooks,
                        int count)
{
	int i;

	for (i = 0; i < count; i++)
		hlist_del_rcu(&hooks[i].list);
}

#endif

