// SPDX-License-Identifier: GPL-2.0-only
/*
 *  add audit function based on extend attribute security.seaudit
 *
 *  This file contains the selinux hooks2 function implementations.
 *
 *  Authors:  zhoupeng, <zhoupeng@uniontech.com>
 *
 *  Copyright (C) 2022 UnionTech Technology, Inc.
 */
#include <linux/socket.h>
#ifndef __LINUX_CONFIG_SECURITY2_H
#define __LINUX_CONFIG_SECURITY2_H

typedef enum {
	SECURITY_E_FILE_CREATE =1,
	SECURITY_E_DIR_CREATE,
	SECURITY_E_SYMLINK_CREATE
}SECURITY_E_TYPE;

//below extended for selinux for audit
#define security_file_close(file) \
	do {\
		security_file_close_real(file);\
	}\
	while(0)

#define security_file_write(file, count) \
	do {\
		security_file_write_real(file,count);\
	}\
	while(0)

#define security_file_read(file, count) \
	do {\
		security_file_read_real(file, count);\
	}\
	while(0)

#define security_process_exit(task) \
	do {\
		security_process_exit_real(task);\
	}\
	while(0)

#define security_file_create(dir_inode, dentry, op) \
	do {\
		security_file_create_real(dir_inode, dentry, op);\
	}\
	while(0)

int security_file_close_real(struct file* file);
int security_process_exit_real(struct task_struct* task);
int security_file_create_real(const struct inode *dir_inode, struct dentry *dentry, SECURITY_E_TYPE op);

int security_file_write_real(struct file* file, size_t count);
int security_file_read_real(struct file* file, size_t count);
int security_socket_getpeersec2_stream(struct socket *sock, char __user *optval,
				       int __user *optlen, unsigned len);
int security_nm_tcp_recvmsg(struct sock* sk, struct msghdr *msg, size_t copied);
int security_nm_udp_recvmsg(struct sock* sk, struct msghdr *msg, size_t copied);
#endif
